/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.client.internal.licensing");

dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");

// aliases and constants
var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var THIS_PACKAGE = com.ibm.team.repository.web.client.internal.licensing;
var SERVICE_URI = "com.ibm.team.repository.service.internal.ILicenseAdminRestService";

//
// Usage examples
//
// Note, all function calls either an alias, e.g.
//     var getLicenseInfo = com.ibm.team.repository.web.client.internal.licensing.getLicenseInfo;
//     getLicenseInfo(...);
// or must be fully qualified...
//     com.ibm.team.repository.web.client.internal.licensing.getLicenseInfo(...);
//

// var SRH = com.ibm.team.repository.web.transport.ServiceResponseHandler;
//getLicenseInfo(new SRH(this, "successMethod", "onRemoteError"));
//getAssignedLicenses(new SRH(this, "successMethod", "onRemoteError"), {userId: "TestJazzGuest1"});
//assignLicense(new SRH(this, "successMethod", "onRemoteError"), {userId: "TestJazzGuest1", licenseId: "com.ibm.team.rtc.developer"});
//unassignLicense(new SRH(this, "successMethod", "onRemoteError"), {userId: "TestJazzGuest1", licenseId: "com.ibm.team.rtc.developer"});

//
// exported constants
//

// note: assumes you are in the admin web UI (<context-root>/admin)
THIS_PACKAGE.FILE_UPLOAD_URI = "service/com.ibm.team.repository.service.internal.license.ILicenseUploadService";
THIS_PACKAGE.SERVER_COUNT_MAX_MSG = "Warning: The server has reached the maximum user limit.  You will not be able to increase the number of users that are assigned a Client Access License type that is counted in the server user limit.";

//
// exported functions
//

// only meant to be used by HoverPopups!
function formatCALTypeDescription(CALType) {
	return "<div><div class=\"com-ibm-team-repository-CALPopup\"><p class=\"firstRow\"><span>Client Access License Type:</span><span class=\"name\">" + CALType.name + "</span></p>" + CALType.description + "</div></div>";
}
THIS_PACKAGE.formatCALTypeDescription = formatCALTypeDescription;

// Returns true if server license is a trial.
function isServerLicenseTrial(licenseInfo) {
	return (licenseInfo.serverLicense && licenseInfo.serverLicense.expirationTime) ? true : false;
}
THIS_PACKAGE.isServerLicenseTrial = isServerLicenseTrial;

// Returns true if there are unassigned licenses of this CAL type and the CAL type is not disabled.
function isCALAvailable(CALType, isTrialServer){
	var count = computeCALAvailability(CALType, isTrialServer);
	return !(count == 0 || count == -2);
}
THIS_PACKAGE.isCALAvailable = isCALAvailable;

// Returns true if the type has non-disabled CALs available and the server will allow assignment to this
//  user.  To allow assignment to this user, either the user must already have CALs assigned or the server 
//  has not reached the maximum number of contributors.
function isCALAvailableForUser(CALType, licenseInfo, userHasNoCALs){
	if (userHasNoCALs && !isServerAcceptingNewLicensedUsers(licenseInfo) && CALType.countedInServerLimit){
		return false; // server can't accept new contributors
	} 
	return isCALAvailable(CALType, isServerLicenseTrial(licenseInfo));
}
THIS_PACKAGE.isCALAvailableForUser = isCALAvailableForUser;

// Returns true if the server has not reached the maximum number of contributors.
function isServerAcceptingNewLicensedUsers(licenseInfo){
	return licenseInfo.serverLicense.maxContributors - licenseInfo.serverLicense.usedContributors >= 1;
}
THIS_PACKAGE.isServerAcceptingNewLicensedUsers = isServerAcceptingNewLicensedUsers;

// Returns the count of available licenses of this type.  
//  Result of -2 means this CAL type is disabled.
//  Result of -1 means the availability is unlimited.
function computeCALAvailability(CALType, isTrialServer) {
	if (CALType.disabledReason) {
		return -2; // CAL is disabled
	}
	if (CALType.total == -1) {
		return CALType.total; //CAL is unlimited
	}
	return isTrialServer ? CALType.max - CALType.used : CALType.total - CALType.used;
}
THIS_PACKAGE.computeCALAvailability = computeCALAvailability;

function sortCALs(types){
	return types.sort(function(a, b){
		// if one type is a functional type and the other isn't, the functional
		// one goes after
		if(a.functional !== b.functional) {
			return a.functional ? 1 : -1;
		}
		// otherwise sort on the name
		var aName = getCALNameInfo(a).toLowerCase();
		var bName = getCALNameInfo(b).toLowerCase();
		return aName > bName ? 1 : -1;
	});
}
THIS_PACKAGE.sortCALs = sortCALs;

// returns the string (Unlimited Availability/Disabled/# available) for availability of a CAL type
function getCALAvailabilityString(CALType, isTrialServer, includeParens) {
	var available = computeCALAvailability(CALType, isTrialServer);
	var label = "";
	if (includeParens) {
		label = "(";
	}
	switch (available){
		case -1:
			label += "Unlimited Availability";
			break;
		case -2:
			label += "Deactivated";
			break;
		default : label += available + " available";
	}
	if (includeParens) {
		label += ")";
	}
	return label;
}
THIS_PACKAGE.getCALAvailabilityString = getCALAvailabilityString;


function getCALNameInfo(CALType) {
	var nameString = "";
	if (CALType.productName) {
		nameString = CALType.productName;
	}
	if (CALType.productName && CALType.name) {
		nameString += " - ";
	}
	if (CALType.name) {
		nameString += CALType.name;
	}
	return nameString;
}
THIS_PACKAGE.getCALNameInfo = getCALNameInfo;

/**
 * Returns an object containing useful UI information for a Client 
 * Access License.
 */
function getClientAccessLicenseInfo(licenseInfo, CALType) {
	var isTrial = isServerLicenseTrial(licenseInfo);
	var availabilityLabel = getCALAvailabilityString(CALType, isTrial, true);
	var simpleAvailabilityString = getCALAvailabilityString(CALType, isTrial, false);
	var available = computeCALAvailability(CALType, isTrial);
	var name = getCALNameInfo(CALType);
	var assignableToUserWithoutCALs = isCALAvailableForUser(CALType, licenseInfo, true);
	var assignableToUserWithCALs = isCALAvailableForUser(CALType, licenseInfo, false);;
	
	return {
		availabilityLabel: availabilityLabel,
		simpleAvailabilityString: simpleAvailabilityString,
		available: available,
		name: name, 
		assignableToUserWithoutCALs: assignableToUserWithoutCALs, 
		assignableToUserWithCALs: assignableToUserWithCALs
	};
}
THIS_PACKAGE.getClientAccessLicenseInfo = getClientAccessLicenseInfo;

/**
 * Returns an object containing useful UI information related to a server
 * license's expiration time.
 * 
 * @param {String} expirationTime the RFC 3339 formatted time stamp contained in
 *  licenseInfo.serverLicense.expirationTime; do not send in null
 */
function getExpirationInfo(expirationTime) {
	
	// format expiration date string
	var expirationDate = dojo.date.stamp.fromISOString(expirationTime);
	var expirationDateString = dojo.date.locale.format(expirationDate, {formatLength: "long", selector: "date" });
	
	// determine if license is expired or not
	var now = new Date();
	var isExpired = now > expirationDate;
	
	// determine how many days remaining or since expiration
	var dayDiff = Math.abs(dojo.date.difference(now, expirationDate, "day"));
	var daysRemaining = isExpired ? null : dayDiff;
	var daysSinceExpiration = isExpired ? dayDiff : null;
	
	// format days remaining string
	var daysStr = dayDiff === 1 ? " day " : " days ";
	if(dayDiff === 0) {
		var daysRemainingString = "Today";
		var daysSinceExpirationString = daysRemainingString;
	} else {
		var daysRemainingString = isExpired ? null : dayDiff + daysStr + "remaining";
		var daysSinceExpirationString = isExpired ? dayDiff + daysStr + "ago" : null;
	}
	
	return {
		daysRemaining: daysRemaining,
		daysRemainingString: daysRemainingString,
		daysSinceExpiration: daysSinceExpiration,
		daysSinceExpirationString: daysSinceExpirationString,
		expirationDateString: expirationDateString,
		isExpired: isExpired
	};
}
THIS_PACKAGE.getExpirationInfo = getExpirationInfo;

/**
 * See ILicenseAdminRestService#getLicenseInfo
 */
function getLicenseInfo(srh) {
	TeamServerClient.invokeService(getServiceRequest("getLicenseInfo"), srh);
}
THIS_PACKAGE.getLicenseInfo = getLicenseInfo;

/**
 * See ILicenseAdminRestService#getAssignedLicenses
 */
function getAssignedLicenses(srh, args) {
	validateArgs("getAssignedLicenses", args, ["userId"]);
	TeamServerClient.invokeService(getServiceRequest("getAssignedLicenses", args), srh);
}
THIS_PACKAGE.getAssignedLicenses = getAssignedLicenses;

var ASSIGN_ARGS = ["userId", "licenseId"];
var ASSIGN_MULTIPLE_ARGS = [ "licenseId", "jsonUserIds" ];

/**
 * See ILicenseAdminRestService#postAssignLicense
 */
function assignLicense(srh, args) {
	validateArgs("assignLicense", args, ASSIGN_ARGS);
	TeamServerClient.invokeService(getServiceRequest("postAssignLicense", args), srh);
}
THIS_PACKAGE.assignLicense = assignLicense;

/**
 * See ILicenseAdminRestService#postAssignLicenseMultiple
 */
function assignLicenseMultiple(srh, args) {
	validateArgs("assignLicenseMultiple", args, ASSIGN_MULTIPLE_ARGS);
	TeamServerClient.invokeService(getServiceRequest("postAssignLicenseMultiple", args), srh);
}
THIS_PACKAGE.assignLicenseMultiple = assignLicenseMultiple;

/**
 * See ILicenseAdminRestService#postUnassignLicense
 */
function unassignLicense(srh, args) {
	validateArgs("unAssignLicense", args, ASSIGN_ARGS);
	TeamServerClient.invokeService(getServiceRequest("postUnassignLicense", args), srh);
}
THIS_PACKAGE.unassignLicense = unassignLicense;

function postLicenseInfo(srh, args) {
	args.jsonMaxCounts = dojo.toJson(args.maxCounts);
	delete args.maxCounts;
	if(args.serverFile === null) {
		delete args.serverFile;
	}
	TeamServerClient.invokeService(getServiceRequest("postLicenseInfo", args), srh);
}
THIS_PACKAGE.postLicenseInfo = postLicenseInfo;

//
// internal functions
//

function getServiceRequest(method, args) {
	if (args) {
		return new ServiceRequest(SERVICE_URI, method, args);
	} else {
		return new ServiceRequest(SERVICE_URI, method);
	}
}

function validateArgs(fName, args, props) {
	for(var i=0; i<props.length; i++) {
		if(!args[props[i]]) {
			throw new Error(fName + "missing required parameter args." + props[i]);
		}
	}
}

})();
